package javax.swing.text;

import java.util.Vector;

public class PlainDocument extends AbstractDocument
{
    public static final String tabSizeAttribute = "tabSize";
    public static final String lineLimitAttribute = "lineLimit";
    private AbstractDocument.AbstractElement defaultRoot;
    private Vector<Element> added;
    private Vector<Element> removed;
    private transient Segment s;
    
    public PlainDocument() {
        this(new GapContent());
    }
    
    public PlainDocument(AbstractDocument.Content content) {
        super(content);
        this.added = new Vector();
        this.removed = new Vector();
        this.putProperty("tabSize", Integer.valueOf(8));
        this.defaultRoot = this.createDefaultRoot();
    }
    
    @Override
    public void insertString(int n, String string, AttributeSet set) throws BadLocationException {
        Object property = this.getProperty("filterNewlines");
        if (property instanceof Boolean && property.equals(Boolean.TRUE) && string != null && string.indexOf(10) >= 0) {
            StringBuilder sb = new StringBuilder(string);
            for (int length = sb.length(), i = 0; i < length; ++i) {
                if (sb.charAt(i) == '\n') {
                    sb.setCharAt(i, ' ');
                }
            }
            string = sb.toString();
        }
        super.insertString(n, string, set);
    }
    
    @Override
    public Element getDefaultRootElement() {
        return this.defaultRoot;
    }
    
    protected AbstractDocument.AbstractElement createDefaultRoot() {
        AbstractDocument.BranchElement branchElement = (AbstractDocument.BranchElement)this.createBranchElement(null, null);
        branchElement.replace(0, 0, new Element[] { this.createLeafElement(branchElement, null, 0, 1) });
        return branchElement;
    }
    
    @Override
    public Element getParagraphElement(int n) {
        Element defaultRootElement = this.getDefaultRootElement();
        return defaultRootElement.getElement(defaultRootElement.getElementIndex(n));
    }
    
    @Override
    protected void insertUpdate(AbstractDocument.DefaultDocumentEvent defaultDocumentEvent, AttributeSet set) {
        this.removed.removeAllElements();
        this.added.removeAllElements();
        AbstractDocument.BranchElement branchElement = (AbstractDocument.BranchElement)this.getDefaultRootElement();
        int offset = defaultDocumentEvent.getOffset();
        int length = defaultDocumentEvent.getLength();
        if (offset > 0) {
            --offset;
            ++length;
        }
        int elementIndex = branchElement.getElementIndex(offset);
        Element element = branchElement.getElement(elementIndex);
        int startOffset = element.getStartOffset();
        int n = element.getEndOffset();
        try {
            if (this.s == null) {
                this.s = new Segment();
            }
            this.getContent().getChars(offset, length, this.s);
            boolean n2 = false;
            for (int i = 0; i < length; ++i) {
                if (this.s.array[this.s.offset + i] == '\n') {
                    int n3 = offset + i + 1;
                    this.added.addElement(this.createLeafElement(branchElement, null, startOffset, n3));
                    startOffset = n3;
                    n2 = true;
                }
            }
            if (!(n2 == 0)) {
                this.removed.addElement(element);
                if (offset + length == n && startOffset != n && elementIndex + 1 < branchElement.getElementCount()) {
                    Element element2 = branchElement.getElement(elementIndex + 1);
                    this.removed.addElement(element2);
                    n = element2.getEndOffset();
                }
                if (startOffset < n) {
                    this.added.addElement(this.createLeafElement(branchElement, null, startOffset, n));
                }
                Element[] anArray = new Element[this.added.size()];
                this.added.copyInto(anArray);
                Element[] anArray2 = new Element[this.removed.size()];
                this.removed.copyInto(anArray2);
                defaultDocumentEvent.addEdit(new AbstractDocument.ElementEdit(branchElement, elementIndex, anArray2, anArray));
                branchElement.replace(elementIndex, anArray2.length, anArray);
            }
            if (Utilities.isComposedTextAttributeDefined(set)) {
                this.insertComposedTextUpdate(defaultDocumentEvent, set);
            }
        }
        catch (BadLocationException ex) {
            throw new Error("Internal error: " + ex.toString());
        }
        super.insertUpdate(defaultDocumentEvent, set);
    }
    
    @Override
    protected void removeUpdate(AbstractDocument.DefaultDocumentEvent defaultDocumentEvent) {
        this.removed.removeAllElements();
        AbstractDocument.BranchElement branchElement = (AbstractDocument.BranchElement)this.getDefaultRootElement();
        int offset = defaultDocumentEvent.getOffset();
        int length = defaultDocumentEvent.getLength();
        int i = branchElement.getElementIndex(offset);
        int elementIndex = branchElement.getElementIndex(offset + length);
        if (i != elementIndex) {
            for (; i <= elementIndex; ++i) {
                this.removed.addElement(branchElement.getElement(i));
            }
            Element[] array2 = { this.createLeafElement(branchElement, null, branchElement.getElement(i).getStartOffset(), branchElement.getElement(elementIndex).getEndOffset()) };
            Element[] anArray = new Element[this.removed.size()];
            this.removed.copyInto(anArray);
            defaultDocumentEvent.addEdit(new AbstractDocument.ElementEdit(branchElement, i, anArray, array2));
            branchElement.replace(i, anArray.length, array2);
        }
        else {
            Element element = branchElement.getElement(i);
            if (!element.isLeaf() && Utilities.isComposedTextElement(element.getElement(element.getElementIndex(offset)))) {
                Element[] array = { this.createLeafElement(branchElement, null, element.getStartOffset(), element.getEndOffset()) };
                defaultDocumentEvent.addEdit(new AbstractDocument.ElementEdit(branchElement, i, new Element[] { element }, array));
                branchElement.replace(i, 1, array);
            }
        }
        super.removeUpdate(defaultDocumentEvent);
    }
    
    private void insertComposedTextUpdate(AbstractDocument.DefaultDocumentEvent defaultDocumentEvent, AttributeSet set) {
        this.added.removeAllElements();
        AbstractDocument.BranchElement branchElement = (AbstractDocument.BranchElement)this.getDefaultRootElement();
        int offset = defaultDocumentEvent.getOffset();
        int length = defaultDocumentEvent.getLength();
        int elementIndex = branchElement.getElementIndex(offset);
        Element element = branchElement.getElement(elementIndex);
        int startOffset = element.getStartOffset();
        int endOffset = element.getEndOffset();
        AbstractDocument.BranchElement[] array = { (AbstractDocument.BranchElement)this.createBranchElement(branchElement, null) };
        Element[] array2 = { element };
        if (startOffset != offset) {
            this.added.addElement(this.createLeafElement(array[0], null, startOffset, offset));
        }
        this.added.addElement(this.createLeafElement(array[0], set, offset, offset + length));
        if (endOffset != offset + length) {
            this.added.addElement(this.createLeafElement(array[0], null, offset + length, endOffset));
        }
        Element[] anArray = new Element[this.added.size()];
        this.added.copyInto(anArray);
        defaultDocumentEvent.addEdit(new AbstractDocument.ElementEdit(branchElement, elementIndex, array2, array));
        array[0].replace(0, 0, anArray);
        branchElement.replace(elementIndex, 1, array);
    }
}
